package handlers

import (
	"context"
	"errors"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/user_management"
)

// NewUserPasswordDeleteHandler returns a UserPasswordDeleteHandler.
func NewUserPasswordDeleteHandler(db *sqlx.DB) *UserPasswordDeleteHandler {
	return &UserPasswordDeleteHandler{
		db: db,
	}
}

// UserPasswordDeleteHandler handles GET /user.
type UserPasswordDeleteHandler struct {
	db *sqlx.DB
}

func (h *UserPasswordDeleteHandler) handle(
	ctx context.Context, username string,
) error {
	log := zerolog.Ctx(ctx)

	q := squirrel.Update(models.UserDBTable).
		Set(models.UserDBPasswordColumn, "").
		Where(squirrel.Eq{models.UserDBUsernameColumn: username})

	tx, err := database.Begin(ctx, h.db)
	if err != nil {
		return err
	}
	defer tx.RollbackIfOpened(*log)
	r, err := database.ExecContext(ctx, tx, q, log)
	if err != nil {
		return err
	}
	count, err := r.RowsAffected()
	if err != nil {
		return err
	}
	if count == 0 {
		return ErrNoSuchUser
	}
	if err := tx.Commit(); err != nil {
		return err
	}

	return nil
}

// Handle handles the incoming request.
func (h *UserPasswordDeleteHandler) Handle(
	params user_management.UserPasswordDeleteParams, principal *models.Principal,
) middleware.Responder {
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	if !principal.IsAdmin {
		log.Error().Msg("only admins can manage user passwords, expect if you are deleting your own password")

		return user_management.NewUserPasswordDeleteUnauthorized()
	}
	if err := h.handle(params.HTTPRequest.Context(), params.Username); err != nil {
		if errors.Is(err, ErrNoSuchUser) {
			return user_management.NewUserPasswordDeleteNotFound()
		}

		return InternalError(err, log, false)
	}

	return user_management.NewUserPasswordDeleteNoContent()
}
